==> grafana-enterprise_12.4.0-22046043985_22046043985_darwin_arm64.tar.gz: grafana-12.4.0-22046043985/public/app/features/dashboard-scene/panel-edit/PanelEditNext/QueryEditor/TransformationEditorRenderer.tsx <==
import { useMemo } from 'react';

import { t } from '@grafana/i18n';
import { Alert } from '@grafana/ui';

import { useActionsContext, useQueryEditorUIContext, useQueryRunnerContext } from './QueryEditorContext';
import { TransformationDebugDisplay } from './TransformationDebugDisplay';
import { TransformationEditor } from './TransformationEditor';
import { TransformationFilterDisplay } from './TransformationFilterDisplay';
import { TransformationHelpDisplay } from './TransformationHelpDisplay';

export function TransformationEditorRenderer() {
  const { data } = useQueryRunnerContext();
  const { selectedTransformation } = useQueryEditorUIContext();
  const { updateTransformation } = useActionsContext();

  // Memoize to avoid recreating potentially large data.series array reference on every render.
  // This prevents unnecessary re-renders and processing in TransformationEditor.
  const inputData = useMemo(() => data?.series ?? [], [data?.series]);

  if (!selectedTransformation) {
    return null;
  }

  if (!selectedTransformation.registryItem?.editor) {
    return (
      <Alert
        severity="error"
        title={t(
          'transformation-editor-renderer.no-transformation-editor-title',
          'Transformation does not have an editor component'
        )}
      />
    );
  }

  return (
    <>
      <TransformationFilterDisplay />
      <TransformationEditor
        inputData={inputData}
        onUpdate={updateTransformation}
        transformation={selectedTransformation}
      />
      <TransformationHelpDisplay />
      <TransformationDebugDisplay />
    </>
  );
}

==> grafana-enterprise_13.0.0-22288738230_22288738230_darwin_arm64.tar.gz: grafana-13.0.0-22288738230/public/app/features/dashboard-scene/panel-edit/PanelEditNext/QueryEditor/TransformationEditorRenderer.tsx <==
import { useMemo } from 'react';

import { t } from '@grafana/i18n';
import { Alert } from '@grafana/ui';

import {
  useActionsContext,
  usePanelContext,
  useQueryEditorUIContext,
  useQueryRunnerContext,
} from './QueryEditorContext';
import { TransformationDebugDisplay } from './TransformationDebugDisplay';
import { TransformationEditor } from './TransformationEditor';
import { TransformationFilterDisplay } from './TransformationFilterDisplay';
import { TransformationHelpDisplay } from './TransformationHelpDisplay';
import { useTransformationInputData } from './hooks/useTransformationInputData';

export function TransformationEditorRenderer() {
  const { data } = useQueryRunnerContext();
  const { selectedTransformation } = useQueryEditorUIContext();
  const { transformations } = usePanelContext();
  const { updateTransformation } = useActionsContext();

  const rawData = useMemo(() => data?.series ?? [], [data]);

  const inputData = useTransformationInputData({
    selectedTransformation,
    allTransformations: transformations,
    rawData,
  });

  if (!selectedTransformation) {
    return null;
  }

  if (!selectedTransformation.registryItem?.editor) {
    return (
      <Alert
        severity="error"
        title={t(
          'transformation-editor-renderer.no-transformation-editor-title',
          'Transformation does not have an editor component'
        )}
      />
    );
  }

  return (
    <>
      <TransformationFilterDisplay />
      <TransformationEditor
        inputData={inputData}
        onUpdate={updateTransformation}
        transformation={selectedTransformation}
      />
      <TransformationHelpDisplay />
      <TransformationDebugDisplay />
    </>
  );
}
